#!/usr/bin/python

# PyPlaneDomains Copyright (C) 2013  Marco Spadini
#
# PyPlaneDomains is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyPlaneDomains is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PyPlaneDomains.  If not, see <http://www.gnu.org/licenses/>.
# 
# 
# About the author:
# Marco Spadini                         
# Dipartimento di Matematica e Informatica,
# Via S. Marta 3, 50139 Firenze,       
# ITALY 
# 
# web: http:\\www.dma.unifi.it\~spadini
# e-mail: marco.spadini@unifi.it
#
#
#  Description:  
#  The purpose of this program is to sketch plane domains defined by a
#  complex sets of inequalities. 
#  The user should provide a reference rectangle R (determined by two opposite 
#  corners, south-west SW=(x0,y0) and north-east NE=(x1,y1)), and a set of 
#  conditions (namely a bunch of inequalities joined by logical connectives as 
#  ".and.", ".or.", ".xor.", or the operator ".not." ). The set of conditions can 
#  be distributed on several lines (all logically linked by ".and.", but do **not**
#  break a single condition over two lines: results are unpredictable).
#  The program tries to sketch the closure of the interior of the set D of those 
#  points (x,y) in the reference rectangle R where the condition holds. (Note that 
#  boundaries, albeit shown, are not necessarily included in the domain D, but do 
#  belong to the closure of D).
#
#  The mathematical notation allowes for Python standard functions
#  (with the addition of ".xor." and "sign").
#


#from pylab import *
import matplotlib.pyplot as plt
import numpy as np
import re
from Tkinter import *
from math import *
import sys, os, string, time

version= '0.6'

root = Tk()
f_entry=[]
blabla = StringVar()
blabla.set('  ..... READY .....  ')

def signum(x):       #useful function sign
  return cmp(x,0)
  
def convFunStr(sf): # defines standard replacements for variables in formula
  f=sf.lower()
  diz = {' +'                      : '',       #kills spaces
         ';'                       : ',',      #semicolons to commas  
         '.and.'                   : ' and ',  # and operator
         '.or.'                    : ' or ',   # or operator
         '.xor.'                   : ' != ',   # xor operator
         '(?<!a)x(?![\[a-zA-Z\(])' : 'x[0]',   #turns lone x into x[0]
         'y'                       : 'x[1]',   #turns y into x[1]
         } 
  for key in diz.keys():
    f = re.sub(key,diz[key],f)
  return f  

# sets output message in GUI
def message(string):
  blabla.set(string)
  root.update_idletasks()
  
#changes user-interface appearance during computation
def busy(event=None): 
  comp1.pack(side='left')
  comp.pack_forget()
  q1.pack(side='left')
  q.pack_forget()
  message('  ....... BUSY .......  ')
  root.update()
  root.update_idletasks()
        
#sets user-interface back to normal after computation
def notbusy(event=None,error=''): 
  comp1.pack_forget()
  comp.pack(side='left')
  q1.pack_forget()
  q.pack(side='left')
  if error:
    message(error)
  else:  
    message('  ...... READY ......  ')
  root.update()
  root.update_idletasks()
  

def incrementa():
  global f_entry,defaultTxt
  f_entry.append(Entry(vtop, width=62))
  f_entry[len(f_entry)-1].pack(side='top')
  f_entry[len(f_entry)-1].focus_set()
  root.update()
  
def decrementa():
  global f_entry
  if len(f_entry)>1:
    f_entry[len(f_entry)-1].pack_forget()
    f_entry.pop()
    root.update()
  else:
    message('Illegal decrement')
    time.sleep(2)
    notbusy()
  f_entry[len(f_entry)-1].focus_set()
    
  
root.title('PyPlaneDomains %s'% version)

toptop = Frame(root); toptop.pack(padx=10,pady=10)
Label(toptop, text="""Draws within the given reference rectangle of opposite corners,\
SW=(x0,y0) and\n\
NE=(x1,y1), the closure of the interior of the set of the points (x,y) where all given\n\
conditions hold.""" ).pack(side='top')


top = Frame(root); top.pack()

vtop = LabelFrame(top, text='Conditions')
vtopt = Frame(vtop,width=100)
vtopt.pack(side='top',expand='true')
cond_add = Button(vtopt, text = 'ADD line', fg='blue', command=incrementa)
cond_add.pack(side=LEFT,pady='5',padx=3)
cond_sub = Button(vtopt, text = 'REMOVE line', fg='blue', command=decrementa)
cond_sub.pack(side=RIGHT,pady='5',padx=3)
vtop.pack()
f_entry.append(Entry(vtop, width=62))
f_entry[0].pack(side='top')
f_entry[0].focus_set()
        
rtop=LabelFrame(top, text='Reference rectangle')
rtop.pack(ipadx=5,ipady=5)

Label(rtop, text='  SW:').pack(side='left')
Label(rtop, text='  x0 =').pack(side='left')
x0_entry = Entry(rtop, width=4)
x0_entry.pack(side='left')
x0_entry.insert('end', '-3.') #default value

Label(rtop, text='  y0 =').pack(side='left')
y0_entry = Entry(rtop, width=4)
y0_entry.pack(side='left')
y0_entry.insert('end', '-3.') #default value

Label(rtop, text='         NE:').pack(side='left')
Label(rtop, text='  x1 =').pack(side='left')
x1_entry = Entry(rtop, width=4)
x1_entry.pack(side='left')
x1_entry.insert('end', '3.') #default value

Label(rtop, text='  y1 =').pack(side='left')
y1_entry = Entry(rtop, width=4)
y1_entry.pack(side='left')
y1_entry.insert('end', '3.') #default value

mid = Frame(root); mid.pack()
Label(mid, text=' % resolution = ').pack(side='left')
n_entry = Scale(mid, orient='horizontal', from_= 1, to=100, tickinterval=99, 
                resolution=1, length= 320)
n_entry.pack(side='left',padx=5)
n_entry.set(20) #default value

bottom = LabelFrame(root, text='System messages')
bottom.pack(ipadx='25')
Label(bottom, textvariable = blabla, fg='red').pack(padx='25')


def calc(event=None):
  busy()
  funzioni = filter(lambda x: x.get().strip() != '',f_entry[::-1])
  try:
    funz = funzioni.pop().get()
  except Exception, nam:
    notbusy(error='Error getting condition')
    print nam
    return
  funz = '('+funz  
  for i in funzioni:
    funz += ') .and. (' + i.get()
  funz += ')'  
  progfun =convFunStr(funz)
  try:  
    x0 = eval(x0_entry.get())
    x1 = eval(x1_entry.get())
    y0 = eval(y0_entry.get())
    y1 = eval(y1_entry.get())
  except Exception, nam:
    notbusy(error=' Some data do not make sense ')
    print nam
    return
  n = n_entry.get()*1000  #number of random points
  
  plt.close()  #closes graphic window, if exists, does noting otherwise
  
  try:
    fnx = compile(progfun, '<string>', 'eval') #compiles function for spped
    def f(x):
      return eval(fnx)
  except Exception, nam:    
    print nam
    notbusy(error='Syntax error')
    return
  
  targetPTx = []
  targetPTy = []
  sw = [ x0 , y0 ]
  xysize = [x1-x0 , y1-y0 ]
  randomPts = sw+xysize*np.random.rand(n+1,2)
  try:
    for i in xrange(n):
      xx0 = randomPts[i] 
      v=f(xx0);
      if v:
        targetPTx.append(xx0[0])
        targetPTy.append(xx0[1])
      xx1 = randomPts[i+1]
      v1=f(xx1);
      if v1:
        targetPTx.append(xx1[0])
        targetPTy.append(xx1[1])
      if (v1 != v):
        xx2=(xx0+xx1)/2.
        for m in [1,2,3,4,5,6,7,8,9,10]:
          v2=f(xx2)
          if (v != v2):
            xx1 = xx2
          else:
            xx0 = xx2
            v=v2
          xx2 = (xx0+xx1)/2.
        targetPTx.append(xx2[0])
        targetPTy.append(xx2[1])
  except Exception, nam:
    print nam
    notbusy(error='Error in expression')
    return
  plt.title(funz)
  plt.plot(targetPTx,targetPTy,',')
  plt.axis([x0, x1, y0, y1])
  plt.grid(True)
  plt.draw()
  notbusy()
  plt.show()
    
 
def smetti(event=None):
  plt.close() #closes graphic window, if exists, does noting otherwise
  print " 'Bye"
  root.destroy()
  root.quit() 

comp = Button(mid, text=' COMPUTE ', relief='raised', fg='blue',command=calc)
comp.pack(side='left')
q = Button(mid, text=' QUIT ', relief='raised', fg='red', command=smetti)
q.pack(side='left')
comp1 = Button(mid, text=' COMPUTE ', relief='sunken', fg='blue', state=DISABLED)
q1 = Button(mid, text=' QUIT ', relief='raised', fg='red', state=DISABLED)

#root.bind('<q>', quit)   
root.bind('<Return>', calc)
root.mainloop()
